import Taro from "@tarojs/taro";
import { View, Image } from "@tarojs/components";
import WidgetCard from "@/components/widgets/widget-card/index";
import { batchGet } from "@/service/character";
import ContactCard from "@/components/contact-card/index";
import style from "./index.module.less";
import IconArrowRight from "@/images/icon_24_right.png";
import { axios } from "taro-axios";
import type {
  ICharacter,
  IEntityDocument,
  TEntityComponent,
} from "@/types/index";
import { useEffect, useState } from "react";

interface Props {
  index: number;
  editMode: boolean;
  component: TEntityComponent;
  components: TEntityComponent[];
  onSwitchChanged: (c: TEntityComponent, checked: boolean) => void;
  onDelete: (c: IEntityDocument) => void;
  onMove: (c: TEntityComponent, direction: number) => void;
  onClick: () => void;
  onStyleChanged: (c: TEntityComponent) => void
}

export default ({
  index,
  editMode,
  component,
  components,
  onSwitchChanged,
  onDelete,
  onMove,
  onClick,
  onStyleChanged,
}: Props) => {
  let cancelTokenSource = axios.CancelToken.source();
  const [contacts, setContacts] = useState<ICharacter[]>([]);
  const fetchList = async () => {
    cancelTokenSource.cancel('Previous request canceled')
    if (!component.data?.value) {
      return;
    }
    // 重新创建一个新的取消令牌源
    cancelTokenSource = axios.CancelToken.source();
    const res = await batchGet(component.data?.value, {
      cancelToken: cancelTokenSource.token
    });

    if (res.code === 0) {
      Taro.nextTick(()=> {
        setContacts(res.data);
      })
    }
  };
  const nav2Profile = (profileId: string)=> {
    if(!editMode){
      Taro.navigateTo({
        url: "/pages/profile/index?profileId=" + profileId,
      });
    }
  }

  // 非编辑模式且垂直平铺，背景显示为白色
  const bgStyle = (component.data?.layout === "mini" || editMode) ? style.bgMini : style.bgFull

  useEffect(() => {
    fetchList();
    return ()=> {
      cancelTokenSource.cancel('Previous request canceled')
    }
  }, [component.id, component?.data?.value]);

  const renderList = () => {
    if (component.data?.layout === "mini") {
      return (
        <View className="grid grid-cols-3 gap-x-18 gap-y-20 px-16">
          {contacts.map((item) => {
            return (
              <View className="flex flex-col gap-8" onClick={() => nav2Profile(item.profileId)}>
                <View className={style.avatar}>
                  {item.avatar && (
                    <Image src={item.avatar} mode='aspectFill' className={style.avatar} lazyLoad></Image>
                  )}
                </View>
                <View className={`truncate ${style.nickName}`}>
                  {item.alias ?? item.name}
                </View>
              </View>
            );
          })}
        </View>
      );
    }

    return (
      <View className={`flex flex-col gap-8 ${editMode ? 'px-16' : ''}`}>
        
        {contacts.map((item) => {
          return (
            <ContactCard
              className={ bgStyle }
              key={item.profileId}
              data={item}
              onClick={() => nav2Profile(item.profileId)}
              renderRight={() => {
                return (
                  <Image
                    src={IconArrowRight}
                    className={style.arrow}
                    mode="aspectFill"
                    lazyLoad
                  ></Image>
                );
              }}
            ></ContactCard>
          );
        })}
      </View>
    );
  };

  // 非编辑模式下样式需要特殊处理，与其它组件不同
  if(!editMode && component.data?.layout !== "mini"){
    return (
      <>
        {renderList()}
      </>
    )
  }
  
  return (
    <>
      <WidgetCard
        key={component.id}
        enabled={component.enabled}
        onSwitchChanged={(checked) => onSwitchChanged(component, checked)}
        index={index}
        components={components}
        editMode={editMode}
        onDelete={() => onDelete(component)}
        onMove={(direction) =>  onMove(component, direction)}
        onClick={onClick}
        onChanged={()=> {
          if(component.data?.layout === 'mini'){
            onStyleChanged({...component, data: {...component.data, layout: 'full'}})
          }else{
            onStyleChanged({...component, data: {...component.data, layout: 'mini'}})
          }
        }}
      >
        {renderList()}
      </WidgetCard>
    </>
  );
};
