import IconMutedVoice from "@/images/icon-muted-voice.png";
import IconPlay from "@/images/icon-play.png";
import IconVoiceWave from "@/images/icon-voice-wave.png";
import { voiceTryout } from '@/service/audio';
import { TServiceAudioModel } from "@/types/index";
import { useBase64AudioPlayer } from '@/utils/audioBase64';
import { Image, View } from "@tarojs/components";
import { useUnload } from "@tarojs/taro";
import { forwardRef, useImperativeHandle, useState } from "react";
import { axios } from "taro-axios";
import style from "./index.module.less";

type ExtendedTServiceAudioModel = TServiceAudioModel & { checked: boolean };

interface Props {
  voiceItem?: ExtendedTServiceAudioModel | null
  voiceName?: string
  voiceNameText?: string
}

export interface IVoicePlayerBar {
  play: (voice: string) => Promise<void>;
}
let cancelTokenSource = axios.CancelToken.source();
export default forwardRef<IVoicePlayerBar, Props>(({voiceItem, voiceName, voiceNameText = '还没有选择音色'}:Props, ref)=> {
  const [playing, setPlaying] = useState(false);
  const { playAudio, stopAudio, onEnded } = useBase64AudioPlayer();

  let loading = false;
  const playVoice = async (voice: string) => {
    // 在发起请求前取消之前的请求
    cancelTokenSource.cancel('Previous request canceled')
    stopAudio();
    try{
      if(loading){
        return
      }
      // 重新创建一个新的取消令牌源
      cancelTokenSource = axios.CancelToken.source();
      loading = true
      const res = await voiceTryout(voice, {
        cancelToken: cancelTokenSource.token
      })
      loading = false
      if(res?.data?.audio){
        playAudio(res.data.audio)
        setPlaying(true)
      }
    }catch(e){
      loading = false
      setPlaying(false)
    }
  }
  const handlePlay = (voice: string)=> {
    if(!playing && voice){
      playVoice(voice)
      setPlaying(true)
    }
  }
  useImperativeHandle(ref, () => {
    return {
      play: playVoice,
    };
  });
  onEnded(() => {
    setPlaying(false)
  })
  useUnload(()=> {
    stopAudio();
    cancelTokenSource.cancel('Previous request canceled')
  })
  const renderPlayerBar = () => {
    // 直接传入 voiceName 与 voiceName 播放自己录制的音色
    // todo: 有时间做成不同的播放 bar
    if(voiceName){
      return(
        <>
          <View className={style.playStatus} onClick={()=> handlePlay(voiceName)}>
            <Image
              src={playing ? IconVoiceWave : IconPlay}
              mode='widthFix'
              className={style.icon}
            ></Image>
          </View>
          <View className='text-14 leading-22 font-medium truncate'>
            {voiceNameText}
          </View>
        </>
      )
    }
    // 播放系统音色
    if (voiceItem) {
      return (
        <>
          <View className={style.playStatus} onClick={()=> handlePlay(voiceItem.voice)}>
            <Image
              src={playing ? IconVoiceWave : IconPlay}
              mode='widthFix'
              className={style.icon}
            ></Image>
          </View>
          <View className='text-14 leading-22 font-medium truncate'>
            {voiceItem?.name} {voiceItem?.gender === 1 ? "男" : "女"} {voiceItem.style}
          </View>
        </>
      );
    }
    // 没有音色的状态
    return (
      <>
        <View className={style.playStatus}>
          <Image
            src={IconMutedVoice}
            mode='widthFix'
            className={style.icon}
          ></Image>
        </View>
        <View className='text-14 leading-22 font-medium text-gray-45'>
          请选择声音{voiceName}
        </View>
      </>
    );
  };

  return <>
   <View className={style.playerBar}>{renderPlayerBar()}</View>
  </>
})
