import { useState } from "react";
import { View } from "@tarojs/components";
import PageCustom from "@/components/page-custom/index";
import NavBarNormal from "@/components/nav-bar-normal/index";
import EmptyData from "@/components/empty-data/index";

import editorStyle from "../editor.module.less";
import style from "./index.module.less";
import WemetaRadioGroup from "@/components/wemeta-radio-group/index";
import WemetaTabs from "@/components/wemeta-tabs/index";
import ButtonCardAdd from "@/components/button-card-add/index";
import MediaItem from "@/components/media-item/index";
import Taro, { useUnload } from "@tarojs/taro";
import { uploadImage } from "@/utils/http";
import { useComponentStore } from "@/store/componentStore";
import { useCharacterStore } from "@/store/characterStore";
import type { TMediaType } from "@/types/index";

interface Props {
  editMode?: boolean;
}

export default function Index({ editMode }: Props) {
  const $instance = Taro.getCurrentInstance();
  const id = $instance.router?.params.id;
  const deltaNum = id ? 1 : 2;
  let isUploading = false;
  let currentComponent = useComponentStore((state) => state.currentComponent);
  const character = useCharacterStore((state) => state.character);
  const { saveComponent } = useComponentStore();
  const loading = useComponentStore((state) => state.loading);

  const [mediaValue, setMediaValue] = useState<TMediaType[]>(
    currentComponent?.data?.media ?? []
  );

  const [radioValue, setRadioValue] = useState(
    currentComponent?.data?.layout ?? "mini"
  );
  const radioList = [
    {
      text: "缩略宫格",
      value: "mini",
      checked: false,
    },
    {
      text: "大图平铺",
      value: "full",
      checked: false,
    },
  ];

  const handleSave = async (_newValue?: string) => {
    if (!mediaValue.length) {
      return;
    }
    if (loading) {
      return;
    }
    const c = {
      data: {
        media: mediaValue,
        layout: radioValue,
      },
      enabled: currentComponent?.enabled ?? true,
      id: currentComponent?.id,
      name: currentComponent?.name ?? "图片/视频",
      characterProfileId:
        currentComponent?.characterProfileId ?? character?.profileId,
      type: "images",
    };

    await saveComponent(c);
  };

  const onRadioChange = (value: string) => {
    setRadioValue(value);
    return false;
  };

  const handleClick = () => {
    Taro.chooseMedia({
      count: 9,
      mediaType: ["image", "video"],
      sourceType: ["album", "camera"],
      async success(res) {
        isUploading = true;
        // tempFilePath可以作为img标签的src属性显示图片
        const tempFilePaths = res.tempFiles;
        const arr: TMediaType[] = [];
        for (let t of tempFilePaths) {
          Taro.showLoading();
          const res = await uploadImage(t.tempFilePath);
          Taro.hideLoading();
          if (res?.code === 0 && res.data) {
            arr.push({
              duration: t.duration,
              height: t.height,
              width: t.width,
              size: t.size,
              url: res.data,
              fileType: t.fileType,
            });
            console.log(arr);
          }else{
            Taro.showModal({
              title: '上传异常',
              content: res?.msg ?? '文件上传失败',
              showCancel: false,
            })
          }
        }
        setMediaValue([...mediaValue, ...arr]);
        isUploading = false;
      },
      fail(err) {
        console.log(err);
        isUploading = false;
      },
    });
  };

  const handleDelete = (current: TMediaType) => {
    const results = mediaValue.filter((item) => item.url !== current.url);
    setMediaValue([...results]);
  };

  const handleNavBack = async () => {
    await handleSave();
  };

  

  const renderList = () => {
    if (mediaValue.length) {
      return (
        <View className="grid grid-cols-3 gap-x-8 gap-y-12 justify-center">
          {mediaValue.map((item) => {
            return (
              <View className="flex">
                <View className={style.imageItem}>
                  <MediaItem media={item}></MediaItem>
                  <View
                    className="iconfont icon-icon_20_delet icon-24 absolute right-8 top-10 text-white"
                    onClick={() => handleDelete(item)}
                  ></View>
                </View>
              </View>
            );
          })}
        </View>
      );
    }
    return <EmptyData></EmptyData>;
  };

  const tabList = [
    {
      key: "1",
      label: "内容",
      children: renderList(),
    },
    {
      key: "2",
      label: "版式",
      children: (
        <View>
          <WemetaRadioGroup
            radioList={radioList}
            value={radioValue}
            onChange={onRadioChange}
          ></WemetaRadioGroup>
        </View>
      ),
    },
  ];

  return (
    <PageCustom isflex={false} bgColor="global-light-green-bg">
      <NavBarNormal
        backText="保存"
        navDelta={deltaNum}
        onNavBack={handleNavBack}
      >
        图片/视频
      </NavBarNormal>

      <View className={editorStyle.container}>
        <ButtonCardAdd
          text="导入图片/视频"
          onClick={handleClick}
        ></ButtonCardAdd>
        <WemetaTabs list={tabList} current="1"></WemetaTabs>
      </View>
    </PageCustom>
  );
}
