import { create } from 'zustand'
import {
  getCharacterById as _getCharacterById,
  getProfileById as _getProfileById,
  listCharacter, saveCharacter,
  cloneVoice as _cloneVoice,
  // greeting as _greeting,
  deleteCharacterById as _deleteCharacterById,
  getVoiceCloneHistory,
} from '@/service/character'
import { ICharacter, TEntityComponent, ModelUserProfile, TEntityVoiceCloneRecord } from '@/types/index'

type TCreateCharacterParams = Omit<ICharacter, 'profileId'> & { profileId?: string };
// 混入自定义样式进 character
const mixinCustomStylesToCharacter = (character: ICharacter) => {
  const customStyled = character?.bgColor ? `
    --color-bg-primary: ${character.bgColor};
    --color-primary: ${character.bgColor};
    ` : `
      --color-bg-primary: ;
      --color-primary: ;
    `
  const customBgImg = character?.bgImg ?? ''
  character = { ...character, customStyled, customBgImg }
  return character
}

export interface CharacterStoreState {
  character: ICharacter | null
  characters: ICharacter[]
  voiceList: TEntityVoiceCloneRecord[]
  components: TEntityComponent[]
  fetchVoiceCloneHistory: (profileId: string) => Promise<TEntityVoiceCloneRecord[]>
  fetchCharacterList: () => Promise<ICharacter[] | null>
  fetchCharacter: (profileId: string) => Promise<void>
  saveCharacter: (c: TCreateCharacterParams) => Promise<{
    code: number;
    data: ICharacter;
    msg: string;
  } | null>
  getProfileById: (profileId: string) => Promise<{
    code: number;
    data: ModelUserProfile;
    msg: string;
  }>
  deleteCharacterById: (profileId: string) => Promise<null>
  getCharacterById: (profileId: string) => Promise<ICharacter | null>
  setCurrentCharacter: (character: ICharacter | null) => void
  cloneVoice: (fileLink: string, key?: string) => Promise<{ code: number; data: string; msg: string; } | undefined>
  // greeting: (profileId:string, configs: any) => Promise<{ audio: string, session: number } | null>
}

export const useCharacterStore = create<CharacterStoreState>((set, get) => ({
  character: null, // 当前角色信息
  characters: [],
  voiceList: [], // 克隆列表
  components: [], // 用户组件列表
  fetchVoiceCloneHistory: async (profileId: string) => {
    const response = await getVoiceCloneHistory(profileId)
    if (response.code === 0) {
      set({ voiceList: response.data })
      return response.data
    }
    return []
  },
  fetchCharacterList: async () => {
    const response = await listCharacter()
    console.log(response?.data)
    if (response.code === 0) {
      const characters = response.data.map((item) => {
        if (item.profileID) {
          return {
            ...item,
            profileId: item.profileID
          }
        }
        const character = mixinCustomStylesToCharacter(item)
        return character
      })
      set({ characters: characters })
      return characters;
    }
    return null
  },
  fetchCharacter: async (profileId: string) => {
    get().getCharacterById(profileId)
  },
  // 保存角色信息, 需要修改什么字段就传什么字段
  saveCharacter: async (c:TCreateCharacterParams) => {
    const response = await saveCharacter(c as ICharacter)
    console.log(response)
    if (response.code !== 0 || !response.data) {
      return null
    }
    const character = mixinCustomStylesToCharacter(response.data)
    get().setCurrentCharacter(character)
    return response
  },
  addCharacter: async (c: ICharacter) => {
    const character = mixinCustomStylesToCharacter(c)
    set((state) => {
      return {
        characters: [...state.characters, character]
      }
    })
  },
  getProfileById: async (profileId: string) => {
    const res = await _getProfileById(profileId)
    if (res.code === 0 && res.data) {
      const character = mixinCustomStylesToCharacter(res.data.character)
      set({ character })
      return res
    }
    return res;
  },
  getCharacterById: async (profileId: string) => {
    const res = await _getCharacterById(profileId)
    if (res.code === 0 && res.data) {
      const character = mixinCustomStylesToCharacter(res.data)
      set({ character })
      return character
    }
    return null;

  },
  deleteCharacterById: async (profileId: string) => {
    const res = await _deleteCharacterById(profileId)
    if (res.code === 0) {
      get().fetchCharacterList()
    }
    return res.data
  },
  setCurrentCharacter: (character: ICharacter | null) => {
    // console.log('set current character:', character)
    set({ character })
  },
  cloneVoice: async (fileLink: string, key?: string) => {
    const profileId = get().character?.profileId;
    if (!profileId) {
      return
    }
    return await _cloneVoice({
      profileId,
      value: fileLink,
      key,
    })
  },
  // greeting: async (profileId, configs: any) => {
  //   if (!profileId) {
  //     return null
  //   }
  //   const res = await _greeting(profileId, configs)
  //   if (res.code === 0) {
  //     return res.data
  //   }
  //   return null
  // }
}))

export const useCurrentCharacter = () => {
  return useCharacterStore((state) => state.character);
}

export const useCharacterStyle = () => {
  const character = useCharacterStore((state) => state.character);
  const customStyled = character?.bgColor ? `
    --color-bg-primary: ${character.bgColor};
    --color-primary: ${character.bgColor};
    ` : `
      --color-bg-primary: ;
      --color-primary: ;
    `
  // 没有自定义颜色时，也需要写 --var 清除一下
  return {
    customStyled,
    customBgImg: character?.bgImg ?? ''
  }
}