import {
  APP_NAME,
  APP_VERSION,
  APP_NAME_TEXT,
  ENABLE_UPGRADE_VIP,
} from "@/config/index";
import { share } from "@/service/character";
import Taro, { FileSystemManager } from "@tarojs/taro";
import { useRef, useCallback } from "react";

const appTokenKey = `${APP_NAME}+${APP_VERSION}token`;

export const getToken = () => {
  return Taro.getStorageSync(appTokenKey);
};
export const setToken = (value: string) => {
  Taro.setStorageSync(appTokenKey, value);
};
export const removeToken = () => {
  Taro.setStorageSync(appTokenKey, "");
};

export const getSuffix = (filePath: string) => {
  const suffix = filePath.split(".").pop() ?? "";
  return suffix;
};

const chars =
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

// Use a lookup table to find the index.
const lookup = typeof Uint8Array === "undefined" ? [] : new Uint8Array(256);
for (let i = 0; i < chars.length; i++) {
  lookup[chars.charCodeAt(i)] = i;
}

export const encode = (arraybuffer: ArrayBuffer): string => {
  let bytes = new Uint8Array(arraybuffer),
    i,
    len = bytes.length,
    base64 = "";

  for (i = 0; i < len; i += 3) {
    base64 += chars[bytes[i] >> 2];
    base64 += chars[((bytes[i] & 3) << 4) | (bytes[i + 1] >> 4)];
    base64 += chars[((bytes[i + 1] & 15) << 2) | (bytes[i + 2] >> 6)];
    base64 += chars[bytes[i + 2] & 63];
  }

  if (len % 3 === 2) {
    base64 = base64.substring(0, base64.length - 1) + "=";
  } else if (len % 3 === 1) {
    base64 = base64.substring(0, base64.length - 2) + "==";
  }

  return base64;
};

export const decode = (base64: string): ArrayBuffer => {
  let bufferLength = base64.length * 0.75,
    len = base64.length,
    i,
    p = 0,
    encoded1,
    encoded2,
    encoded3,
    encoded4;

  if (base64[base64.length - 1] === "=") {
    bufferLength--;
    if (base64[base64.length - 2] === "=") {
      bufferLength--;
    }
  }

  const arraybuffer = new ArrayBuffer(bufferLength),
    bytes = new Uint8Array(arraybuffer);

  for (i = 0; i < len; i += 4) {
    encoded1 = lookup[base64.charCodeAt(i)];
    encoded2 = lookup[base64.charCodeAt(i + 1)];
    encoded3 = lookup[base64.charCodeAt(i + 2)];
    encoded4 = lookup[base64.charCodeAt(i + 3)];

    bytes[p++] = (encoded1 << 2) | (encoded2 >> 4);
    bytes[p++] = ((encoded2 & 15) << 4) | (encoded3 >> 2);
    bytes[p++] = ((encoded3 & 3) << 6) | (encoded4 & 63);
  }

  return arraybuffer;
};

export function throttle(func: Function, delay: number): Function {
  let timer: NodeJS.Timeout | null = null;
  return function (...args: any[]) {
    if (!timer) {
      func.apply(this, args);
      timer = setTimeout(() => {
        timer = null;
      }, delay);
    }
  };
}

export function debounce(func: Function, delay: number): Function {
  let timer: NodeJS.Timeout | null = null;
  return function (...args: any[]) {
    if (timer) {
      clearTimeout(timer);
    }
    timer = setTimeout(() => {
      func.apply(this, args);
      timer = null;
    }, delay);
  };
}

/**
 * 自定义Hook实现debounce功能
 * @param value 要延迟的值
 * @param delay 延迟时间（毫秒）
 * @returns 经过debounce处理后的值
 */
export function useDebounce(fn: () => void, delay: number) {
  const timer = useRef<null | ReturnType<typeof setTimeout>>(null);
  return () => {
    timer.current && clearTimeout(timer.current);
    timer.current = setTimeout(fn, delay);
  };
}

// 修改 useDebounce 使其支持任意参数
export function useDebounceWithParams<T extends (...args: any[]) => any>(
  fn: T,
  delay: number
): T {
  const timerRef = useRef<NodeJS.Timeout | null>(null);

  return useCallback(
    (...args: Parameters<T>) => {
      if (timerRef.current) {
        clearTimeout(timerRef.current);
      }

      timerRef.current = setTimeout(() => {
        fn(...args);
      }, delay);
    },
    [fn, delay]
  ) as T;
}

export const getSharePromise = async (profileId: string) => {
  const res = await share(profileId, "wechat");
  console.log(res, "shared referralCode:", res.data.referralCode);
  if (res.code === 0) {
    return {
      title: `${APP_NAME_TEXT} | 你的智能名片`,
      path: `/pages/profile/index?isShared=true&profileId=${profileId}&referralCode=${res.data.referralCode}`,
    };
  }
  return {
    path: `/pages/index/index`,
  };
};

export const formatDate = (date: Date): string => {
  const year = date.getFullYear();
  const month = String(date.getMonth() + 1).padStart(2, "0");
  const day = String(date.getDate()).padStart(2, "0");
  return `${year}-${month}-${day}`;
};

export const formatDateFull = (date: Date): string => {
  const year = date.getFullYear();
  const month = String(date.getMonth() + 1).padStart(2, "0"); // 月份从 0 开始，所以加 1
  const day = String(date.getDate()).padStart(2, "0");
  const hours = String(date.getHours()).padStart(2, "0");
  const minutes = String(date.getMinutes()).padStart(2, "0");

  return `${year}-${month}-${day} ${hours}:${minutes}`;
};

export const formatSeconds = (seconds: number): string => {
  // 计算分钟数
  const minutes = Math.floor(seconds / 60);
  // 计算剩余的秒数
  const remainingSeconds = seconds % 60;

  // 确保分钟和秒都以两位数的形式显示
  const formattedMinutes = String(minutes).padStart(2, "0");
  const formattedSeconds = String(remainingSeconds).padStart(2, "0");

  // 返回格式化的字符串
  return `${formattedMinutes}:${formattedSeconds}`;
};

// 是否是中文字符
export function isChineseChatacter(str: string): boolean {
  const code = str.charCodeAt(0);
  if (
    (code >= 0x4e00 && code <= 0x9fff) ||
    (code >= 0x3400 && code <= 0x4dbf) ||
    (code >= 0x20000 && code <= 0x2a6df) ||
    (code >= 0x2a700 && code <= 0x2b73f) ||
    (code >= 0x2b740 && code <= 0x2b81f) ||
    (code >= 0x2b820 && code <= 0x2ceaf) ||
    (code >= 0xf900 && code <= 0xfaff) ||
    (code >= 0x2f800 && code <= 0x2fa1f)
  ) {
    return true;
  }

  return false;
}

export function countCharacters(str: string): number {
  let count = 0;
  const arr = Array.from(str);
  for (let i = 0, j = arr.length; i < j; i++) {
    // 判断是否为中文字符
    if (isChineseChatacter(arr[i])) {
      count += 1;
    } else {
      // 英文字符每两个算一个
      count += 0.5;
    }
  }

  return Math.ceil(count);
}

export const getStrByMaxLength = (str: string, length: number) => {
  const arr = Array.from(str);
  let result = "";
  let l = 0;
  for (let i = 0, j = arr.length; i < j; i++) {
    const s = arr[i];
    if (isChineseChatacter(s)) {
      l += 1;
    } else {
      l += 0.5;
    }
    result += s;
    console.log(l, length);
    if (Math.ceil(l) >= length) {
      return result;
    }
  }
  return result;
};

export const navToWebView = (url: string) => {
  Taro.navigateTo({
    url: `/pages/webview/index?url=${encodeURIComponent(url)}`,
  });
};

export const getCanvasTempPath = (
  canvas: any,
  canvasId: string
): Promise<string> => {
  return new Promise((resolve, reject) => {
    Taro.canvasToTempFilePath(
      {
        x: 0,
        y: 0,
        fileType: "jpg",
        quality: 0.8,
        canvas: canvas,
        canvasId: canvasId,
        success: (res) => {
          console.log(res, res.tempFilePath);
          resolve(res.tempFilePath);
          // wx.showToast({
          // 	title: 'canvas转图成功' + res.tempFilePath,
          // 	duration: 3000
          // })
        },
        fail: (res) => {
          console.log(res);
          Taro.showToast({
            title: "canvas转图片失败" + JSON.stringify(res),
            icon: "none",
            duration: 3000,
          });
          reject("");
        },
      },
      this
    );
  });
};

export const savePicture = async (tmpPath: string): Promise<boolean> => {
  return new Promise((resove, reject) => {
    const params = {
      filePath: tmpPath,
      success() {
        console.log("save success");
        resove(true);
      },
      fail(error: any) {
        console.log(error);
        reject(false);
      },
    };
    Taro.saveImageToPhotosAlbum(params);
  });
};

export const getCloneVoiceIdentifier = (number: number): string => {
  const formattedNumber = number.toString().padStart(2, "0");

  return `克隆声音${formattedNumber}`;
};

export const navToVip = () => {
  if (ENABLE_UPGRADE_VIP) {
    Taro.navigateTo({ url: "/pages/vip/index" });
    return;
  }
  Taro.showModal({
    content: "此功能暂不可用",
    showCancel: false,
  });
};

export const getTempFileContent = <T>(
  tmpPath: string,
  encoding?: keyof FileSystemManager.Encoding
): Promise<T> => {
  return new Promise((resolve, reject) => {
    const fs = Taro.getFileSystemManager();
    fs.readFile({
      filePath: tmpPath,
      encoding: encoding,
      success: (res) => {
        const fileContent = res.data as T;
        console.log(
          "%c [ fileContent ]: ",
          "color: #bf2c9f; background: pink; font-size: 13px;",
          fileContent
        );
        resolve(fileContent);
      },
      fail: (err) => {
        console.error("读取文件失败", err);
        reject(err);
      },
    });
  });
};


export const delay = (ms: number) => new Promise(resolve => setTimeout(resolve, ms));

export function generateRandomId(): string {
  const timestamp = Date.now().toString(36); // 当前时间戳转换为 base-36 字符串
  const randomNum = Math.random().toString(36).substring(2, 8); // 生成随机数并转换为 base-36 字符串
  return `${timestamp}-${randomNum}`; // 组合时间戳和随机数
}